unit hiFileInfo;
// Author: Sashaoli
// Version: 1.0
// Date: September 2017

interface

uses Kol,Windows,Share,Debug;

//type             
//  LongRec = packed record
//    Lo, Hi: Word;
//  end;

type
  THIFileInfo = class(TDebug)
   private
    ResultStr:string;  
    VerInfoResult: array[0..14] of string;
    function CurrentFileInfo(const aFileName: string): Boolean;
    function FileAge(const aFileName: string): TDateTime;
    function FomatMask(aMask:string):string;
    procedure WriteDateTime(aFileName:string);
   public
    _prop_FileName:string;
    _prop_Mask:string;

    _data_FileName:THI_Event;
    _event_onGetInfo:THI_Event;

    procedure _work_doGetInfo(var _Data:TData; Index:word);
    procedure _var_Result(var _Data:TData; Index:word);
  end;

implementation

const VerInfoName:array[0..14] of string = ('CompanyName',
                                           'FileDescription',
                                           'FileVersion',
                                           'InternalName',
                                           'LegalCopyRight',
                                           'LegalTradeMarks',
                                           'OriginalFileName',
                                           'ProductName',
                                           'ProductVersion',
                                           'Comments',
                                           'SpecialBuild',
                                           'PrivateBuild',
                                           'E-mail','','');

      VarName: array[0..14] of string = ('%Company','%Descrip',
                                         '%FVersion','%IName',
                                         '%Copyr','%TMarks',
                                         '%OFName','%PName',
                                         '%PVersion','%Comm',
                                         '%SBuild','%PBuild',
                                         '%Email','%Date','%Time');                                         

function THIFileInfo.CurrentFileInfo(const aFileName: string): Boolean;
var i:Integer;
    curinfo: string;
    dump: DWORD;
    size: integer;
    buffer: PChar;
    VersionPointer, TransBuffer: PChar;
    Temp: integer;
    CalcLangCharSet: string;
begin
  Result:= False;
  size := GetFileVersionInfoSize(PChar(aFileName), dump);
  buffer := AllocMem(size+1);
  try
    GetFileVersionInfo(PChar(aFileName), 0, size, buffer);

    VerQueryValue(buffer, '\VarFileInfo\Translation', pointer(TransBuffer),dump);
    if dump >= 4 then
    begin
      temp:=0;
      StrLCopy(@temp, TransBuffer, 2);
      CalcLangCharSet:=Int2Hex(temp, 4);
      StrLCopy(@temp, TransBuffer+2, 2);
      CalcLangCharSet := CalcLangCharSet+Int2Hex(temp, 4);
    end;
    
    for i:= 0 to 12 do begin 
    VerQueryValue(buffer, pchar('\StringFileInfo\'+CalcLangCharSet+'\'+VerInfoName[i]), pointer(VersionPointer), dump);
    if (dump > 1) then
    begin
      SetLength(curinfo, dump);
      StrLCopy(Pchar(curinfo), VersionPointer, dump);
      VerInfoResult[i]:=TrimRight(curinfo);
    end
    else
      VerInfoResult[i]:= 'n/a';
    end;
    WriteDateTime(aFileName);
    Result:= True;   
  finally
    FreeMem(Buffer);
  end;

end;

function THIFileInfo.FileAge(const aFileName: string): TDateTime;
var Handle: THandle;
    FindData: TWin32FindData;
    LocalFileTime: TFileTime;
    ST:TSystemTime;
begin
  Handle := FindFirstFile(PChar(aFileName), FindData);
  if Handle <> INVALID_HANDLE_VALUE then
  begin  
    Windows.FindClose(Handle);
    if (FindData.dwFileAttributes and FILE_ATTRIBUTE_DIRECTORY) = 0 then
    begin
      
      FileTimeToLocalFileTime(FindData.ftLastWriteTime, LocalFileTime);
//      if FileTimeToDosDateTime(LocalFileTime, LongRec(Result).Hi,
//        LongRec(Result).Lo) then Exit;
      FileTimeToSystemTime(LocalFileTime,ST);
      SystemTime2DateTime(ST,Result);      
    end;
  end else Result := -1;
end;

procedure THIFileInfo.WriteDateTime(aFileName:string);
var FT: TDateTime;
begin 
  FT:=FileAge(aFileName);  
  if FT <> -1.0 then
  begin   
    VerInfoResult[13]:= Date2StrFmt('yyyy/MM/dd',FT);
    VerInfoResult[14]:= Time2StrFmt('HH:mm',FT);
  end;
end;

function THIFileInfo.FomatMask(aMask:string):string;
var i:Integer;
begin
  Result:= aMask;
  for i:=0 to 14 do 
    Replace(Result,VarName[i],VerInfoResult[i]);
end;


procedure THIFileInfo._work_doGetInfo;
Var fFileName,fExt:string;
begin
  fFileName:=ReadString(_Data,_data_FileName,_prop_FileName);
  If not FileExists(fFileName) then Exit;
  fExt:= LowerCase(ExtractFileExt(fFileName)); 
  if fExt <> '.exe' then
    if fExt <> '.dll' then Exit; 
  if CurrentFileInfo(fFileName) then ResultStr:= FomatMask(_prop_Mask);
  _hi_CreateEvent(_Data,@_event_onGetInfo,ResultStr);
end;

procedure THIFileInfo._var_Result;
begin
  dtString(_Data,ResultStr);
end;

end.
